/**
 * Album Cover Grabber - Grabs album covers automatically off the internet
 * Copyright (C) 2002  Kevin M Schmidt This program is free software; you can
 * redistribute it and/or modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version. This program is
 * distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE.  See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc., 59
 * Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.bluedragontavern.albumgrabber;

import com.bluedragontavern.albumgrabber.allmusic.*;
import com.bluedragontavern.albumgrabber.google.*;
import com.bluedragontavern.albumgrabber.gui.*;


/**
 * Thread to search for a given album
 */
public class SearchThread
	extends Thread
	{
	private ItemToGrab item;

	/**
	 * Constructor
	 *
	 * @param album DOCUMENT ME!
	 */
	public SearchThread(ItemToGrab item)
		{
		this.item = item;
		}

	/**
	 * Run the search
	 */
	public void run()
		{
		try
			{
			if (item instanceof Album)
				{
				//If we are searching artist pics from google
				if (Mp3RootChooser.getInstance().isGoogeAlbumSearch())
					{
					item.setThumbnails(ImageSearch.getThumbnails(
													 ((Album)item).toStringSearch(), 0));

					//If thumbnails present, mark as found
					if (item.hasThumbnails())
						{
						//Increment num artists sucessfully located
						Report.getInstance().incNumArtistsLocated();
						item.found();
						}
					}
				else //Search artist pics from allmusic
					item = AlbumSearch.search((Album)item);
				}
			else if (item instanceof Artist)
				{
				//If we are searching artist pics from google
				if (Mp3RootChooser.getInstance().isGoogeArtistSearch())
					{
					item.setThumbnails(ImageSearch.getThumbnails(
													 ((Artist)item).getSearchArtistName(),
													 0));

					//If thumbnails present, mark as found
					if (item.hasThumbnails())
						{
						//Increment num artists sucessfully located
						Report.getInstance().incNumArtistsLocated();
						item.found();
						}
					}
				else //Search artist pics from allmusic
					item = ArtistSearch.search((Artist)item);
				}

			if (item.isFound())
				ImageChooserMgr.getInstance().addItem(item);
			}
		catch (Exception ex)
			{
			System.out.println(ex);
			}
		finally
			{
			SearchThreadMgr.getInstance().decrementRunningThreadCount();
			}
		}
	}